#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <stdarg.h>
#include <assert.h>
#include "odicon.h"
#include "SFMT.h"

int oracle_call, improve_call, iteration;
static double time;
double cpu_time_limit = 0;

void counter_reset(void) {
    oracle_call = improve_call = iteration = 0;
    time = cpu_time();
}

void counter_print(void) {
    printf("#oracle call = %d, ", oracle_call);
    /*printf("#improved call = %d, ", improve_call);*/
    printf("#iteration = %d, ", iteration);
    printf("CPU time = %.2f(sec)\n\n", cpu_time() - time);
}

int is_feasible(int dim, double func(int,int*), int x[]) {
    if (func(dim, x) == PLUS_INFINITY) {
        printf("A feasible solution is needed.\n");
        return FALSE;
    }
    return TRUE;
}

void status_print(int dim, double func(int,int*), int x[]) {
    printf("f(x) = %3g,  x = ", func(dim, x));
    ivector_print(dim, x);  printf("\n");
    if (cpu_time_limit > 0 && cpu_time() - time > cpu_time_limit) {
        printf("Give up! Exceeded CPU time limit (%g sec).\n", cpu_time_limit);
        exit(1);
    }
}

#define EPS (1.0e-10)

/* compare double with error rounding */
int fcmp(double a, double b) {
    double diff;

    diff = a-b;
    if ( fabs(diff) < EPS ) return 0;
    return ( (diff>0) ? 1 : -1 );
}

int iround(double x) {  return floor(x + 0.5);  }

void ivector_round(const int dim, int *x, double *y) {
    int i;
    for (i=0; i<dim; i++) x[i] = iround(y[i]);
}

double int_func(int dim, double double_func(int,double*), int x[]) {
    int i;
    double y[5000];

    assert( dim <= sizeof(y)/sizeof(y[0]) );
    for (i=0; i<dim; i++) y[i] = x[i];
    return double_func(dim, y);
}


double intint_func(int dim, double double_func(int,double*), int x[]) {
    return ceil(int_func(dim, double_func, x));
}


void *xmalloc(size_t size) {
    void *s;

    if ( size == 0 ) size++;
    if ( (s=malloc(size)) == NULL ) {
        fprintf( stderr, "malloc : Not enough memory.\n" );
        exit( EXIT_FAILURE );
    }
    return s;
}


int *ivector_new(const int dim, ...) {
    int i;
    int *x;
    va_list ap;

    x = xmalloc(sizeof(x[0]) * dim);
    va_start(ap, dim);
    for (i=0; i<dim && i<30; i++) x[i] = va_arg(ap, int);
    va_end(ap);
    return x;
}

double *fvector_new(const int dim, ...) {
    int i;
    double *x;
    va_list ap;

    x = xmalloc(sizeof(x[0]) * dim);
    va_start(ap, dim);
    for (i=0; i<dim && i<30; i++) x[i] = va_arg(ap, double);
    va_end(ap);
    return x;
}

void ivector_print(const int dim, const int x[]) {
    int i;

    if (dim == 0) { printf("[]"); return; }
    printf("[%d", x[0]);
    for (i=1; i<dim; i++) printf(", %d", x[i]);
    printf("]");
}


void fvector_print(const int dim, const double x[]) {
    int i;

    if (dim == 0) { printf("[]"); return; }
    printf("[%3g", x[0]);
    for (i=1; i<dim; i++) printf(", %3g", x[i]);
    printf("]");
}


int *ivector_copy(const int dim, int *dst, const int *src) {
    return memcpy(dst, src, sizeof(src[0]) * dim);
}

double *fvector_copy(const int dim, double *dst, const double *src) {
    return memcpy(dst, src, sizeof(src[0]) * dim);
}

int ivector_comp(const int dim, int *dst, const int *src) {
    return memcmp(dst, src, sizeof(src[0]) * dim);
}

int fvector_comp(const int dim, double *dst, const double *src) {
    int i, ret;

    for (i=0; i<dim; i++) {
        ret = fcmp(dst[i], src[i]);
        if (ret != 0) return ret;
    }
    return 0;
}


int **imatrix_new(int m, int n) {
    int **a;
    int i, size1, size2, unit1, unit2;
    char *p;

    unit1 = sizeof(*a);
    unit2 = sizeof(**a);

    size1 = unit1 * m;
    size2 = unit2 * n;

    size1 = (size1+unit2-1) / unit2 * unit2; /* adjust alignment */

    p = xmalloc(size1 + size2 * m);

    a = (int**)p;  p += size1;
    for (i=0; i<m; i++) {
        a[i] = (int*)p;  p += size2;
    }
    return a;
}


double **fmatrix_new(int m, int n) {
    double **a;
    int i, size1, size2, unit1, unit2;
    char *p;

    unit1 = sizeof(*a);
    unit2 = sizeof(**a);

    size1 = unit1 * m;
    size2 = unit2 * n;

    size1 = (size1+unit2-1) / unit2 * unit2; /* adjust alignment */

    p = xmalloc(size1 + size2 * m);

    a = (double**)p;  p += size1;
    for (i=0; i<m; i++) {
        a[i] = (double*)p;  p += size2;
    }
    return a;
}


#define DELIMITER " \t\n,;{}[]"

int ivector_scan(const int dim, int *a) {
    int i;
    char *s, buff[BUFSIZ];

    if (fgets(buff, BUFSIZ, stdin) == NULL) return FALSE;
    i = 0;
    for (s=strtok(buff,DELIMITER); s!=NULL; s=strtok(NULL,DELIMITER)) {
        if (*s == '\0') continue;
        if (i < dim) a[i++] = (int)atof(s);
        else break;
    }
    return TRUE;
}

int fvector_scan(const int dim, double *a) {
    int i;
    char *s, buff[BUFSIZ];

    if (fgets(buff, BUFSIZ, stdin) == NULL) return FALSE;
    i = 0;
    for (s=strtok(buff,DELIMITER); s!=NULL; s=strtok(NULL,DELIMITER)) {
        if (*s == '\0') continue;
        if (i < dim) a[i++] = atof(s);
        else break;
    }
    return TRUE;
}

 /* triangle matrix */
int imatrix_scan(const int dim, int **aa) {
    int i;

    for (i=1; i<dim; i++) {  /* we ignore aa[0][0] */
        if (!ivector_scan(i, aa[i])) return FALSE;
    }
    return TRUE;
}

 /* triangle matrix */
int fmatrix_scan(const int dim, double **aa) {
    int i;

    for (i=1; i<dim; i++) {  /* we ignore aa[0][0] */
        if (!fvector_scan(i, aa[i])) return FALSE;
    }
    return TRUE;
}

void fmatrix_print(const int dim, double **aa) {
    int i;

    for (i=0; i<dim; i++) {
        fvector_print(dim, aa[i]);
    }
}


void mt_initialize(long seed) {
    static int initialized = FALSE;
    if (!initialized) {
        init_gen_rand(seed);
        initialized = TRUE;
    }
}

/* min <= irandom() <= max */
int irandom(int min, int max) {
    mt_initialize(123);
    return min + (int)((double)gen_rand32() / 4294967296.0 * (max-min+1));
}

/* min <= frandom() <= max */
double frandom(double min, double max) {
    mt_initialize(123);
    return min + ((double)gen_rand32() / 4294967295.0 * (max-min));
}
